#include "credentialdialog.h"
#include "ui_credentialdialog.h"

#include "definitions.h"
#include "keychain.h"

#include <QApplication>
#include <QEventLoop>
#include <QMessageBox>
#include <QTimer>

#include <qt5keychain/keychain.h>

#ifdef QT_DEBUG
#include <iostream>
#endif

#define KEYCHAIN_SERVICE PRODUCT_NAME

bool CredentialDialog::allWriteOk = true;
QString CredentialDialog::errorString = "";

CredentialDialog::CredentialDialog(QWidget *parent) :
	QDialog(parent),
	ui(new Ui::CredentialDialog)
{
	ui->setupUi(this);
}

CredentialDialog::~CredentialDialog()
{
	delete ui;
}

/**
 * @brief Finish the writing of one password writing job
 * @param job The finished job
 *
 * This method is the finish slot for the qkeychain password jobs.
 * If an error occoured while finishing a job we set the golbal
 * non-error flag to false and append the error message of the
 * job to the global error string. Else we only set the global
 * non-error flag to ((prevoius value) AND true).
 */
inline void CredentialDialog::writeFinished(QKeychain::Job &job, QEventLoop &loop)
{
	if (job.error())
	{
		CredentialDialog::allWriteOk &= false;
		CredentialDialog::errorString.append("\n\t- ").append(qPrintable(job.errorString()));
#ifdef QT_DEBUG
		std::cerr << "Write failed " << qPrintable(job.errorString()) << std::endl;
#endif

	}
	else
	{
#ifdef QT_DEBUG
		std::cout << "Password write OK" << std::endl;
#endif
		CredentialDialog::allWriteOk &= true;
	}
	loop.exit();
}

/**
 * @brief User hits the "save" button
 *
 * This method tries to wirte all the keys into the key storage.
 * If this was sucessfull, it sets the result of this QDialog
 * instance to accepted. Otherwise the reuslt of this QDialog
 * instance will be rejected and an error message will be shown.
 */
void CredentialDialog::btnOkClicked()
{

	QKeychain::WritePasswordJob qkeywriteDatabase(KEYCHAIN_SERVICE);
	QKeychain::WritePasswordJob qkeywritePassword(KEYCHAIN_SERVICE);
	QKeychain::WritePasswordJob qkeywriteServer(KEYCHAIN_SERVICE);
	QKeychain::WritePasswordJob qkeywriteUser(KEYCHAIN_SERVICE);

	qkeywriteDatabase.setAutoDelete(false);
	qkeywritePassword.setAutoDelete(false);
	qkeywriteServer.setAutoDelete(false);
	qkeywriteUser.setAutoDelete(false);

	QEventLoop serverLoop;
	QEventLoop databaseLoop;
	QEventLoop userLoop;
	QEventLoop passwordLoop;

	QObject::connect(
				&qkeywriteDatabase,
				&QKeychain::WritePasswordJob::finished,
				&databaseLoop,
				[&qkeywriteDatabase, &databaseLoop](){ CredentialDialog::writeFinished(qkeywriteDatabase, databaseLoop); });
	QObject::connect(
				&qkeywritePassword,
				&QKeychain::WritePasswordJob::finished,
				&passwordLoop,
				[&qkeywritePassword, &passwordLoop](){ CredentialDialog::writeFinished(qkeywritePassword, passwordLoop); });
	QObject::connect(
			&qkeywriteServer,
			&QKeychain::WritePasswordJob::finished,
			&serverLoop,
			[&qkeywriteServer, &serverLoop](){ CredentialDialog::writeFinished(qkeywriteServer, serverLoop); });
	QObject::connect(
			&qkeywriteUser,
			&QKeychain::WritePasswordJob::finished,
			&userLoop,
			[&qkeywriteUser, &userLoop](){ CredentialDialog::writeFinished(qkeywriteUser, userLoop); });

	qkeywriteDatabase.setKey(KeyChain::database_key);
	qkeywriteDatabase.setTextData(ui->txtDatabase->text());
	qkeywriteDatabase.start();
	databaseLoop.exec();

	qkeywritePassword.setKey(KeyChain::password_key);
	qkeywritePassword.setTextData(ui->txtPassword->text());
	qkeywritePassword.start();
	passwordLoop.exec();

	qkeywriteServer.setKey(KeyChain::server_key);
	qkeywriteServer.setTextData(ui->txtServer->text());
	qkeywriteServer.start();
	serverLoop.exec();

	qkeywriteUser.setKey(KeyChain::username_key);
	qkeywriteUser.setTextData(ui->txtUser->text());
	qkeywriteUser.start();
	userLoop.exec();

	if (CredentialDialog::allWriteOk)
	{
		accept();
		this->close();
	}
	else
	{
		QMessageBox::critical(this,
							  QApplication::translate("Definitions", PRODUCT_NAME),
							  QApplication::translate("Definitions", CREDENTIALS_SAVE_FAILED).arg(this->errorString));

		reject();
		this->close();
	}
}

/**
 * @brief User has hit the abort button
 */
void CredentialDialog::btnAbortClicked()
{
	this->setResult(QDialog::Rejected);
	this->close();
}

void CredentialDialog::showInfo()
{
	QString message = tr("No credentials were found.\nPlease enter the credentials for the database.");
	QMessageBox::warning(
				this,
				QApplication::translate("Definitions", PRODUCT_NAME),
				message);
}

void CredentialDialog::showEvent(QShowEvent *event)
{
	QTimer::singleShot(50, this, &CredentialDialog::showInfo);
}
