#include "dataworker.h"

#include "album.h"
#include "smallalbum.h"

#include <QList>
#include <QMap>
#include <QString>

#include <UmlauteWeb/Qt.hpp>

/**
 * @brief Convert a single string vector to an @see Album
 * @param entry single database entry
 * @return \p entry as \see Album
 */
Album* DataWorker::getAlbum(QVector<QString> *entry)
{
	bool parsingOK = true;

	Umlaute_Web::Qt::decode((*entry)[0]);	// Title
	Umlaute_Web::Qt::decode((*entry)[1]);	// Interpret
	Umlaute_Web::Qt::decode((*entry)[4]);	// Genre
	Umlaute_Web::Qt::decode((*entry)[6]);	// Location
	Umlaute_Web::Qt::decode((*entry)[7]);	// Owner

	int year = entry->at(2).toInt(&parsingOK);

	bool isCD_DVD = entry->at(5) == "CD + DVD";

	Album *album = parsingOK ? new Album
	{
			entry->at(3),	// ean
			entry->at(0),	// title
			entry->at(1),	// interpret
			year,
			entry->at(4),	// genre
			entry->at(6),	// location
			isCD_DVD,
			entry->at(7)	// owner
	} : nullptr;
	return album;

}

/**
 * @brief Convert a single string vector to an \see SmallAlbum
 * @param entry The single database entry
 * @return \p entry as \see SmallAlbum
 *
 * It must be ensured that \p entry has at least three entries and they can be interpreted as ean, title and interpret
 * (in this order).
 */
SmallAlbum* DataWorker::getSmallAlbum(QVector<QString> *entry)
{
	Umlaute_Web::Qt::decode((*entry)[1]);	// Title
	Umlaute_Web::Qt::decode((*entry)[2]);	// Interpret

	SmallAlbum *album = new SmallAlbum
	{
		entry->at(1),	// title
		entry->at(2),	// interpret
		entry->at(0)	// ean
	};

	return album;
}

/**
 * @brief Converts string vector from database to list of @see Album
 * @param entries Entries given from database
 * @return \p entries as @see Album
 */
QList<Album*>* DataWorker::getAlbums(QList<QVector<QString>*> *entries)
{
	QList<Album*> *albums;
	if (entries->count() > 0)
	{
		albums = new QList<Album*>();
		for (auto &&item : *entries)
		{
			Album *album = getAlbum(item);
			albums->append(album);
		}
	}
	else { albums = nullptr; }

	return albums;
}

/**
 * @brief Converts string vector from database to list of @see SmallAlbum
 * @param entries Entries given from database
 * @return \p entries as @see SmallAlbum
 */
QList<SmallAlbum*>* DataWorker::getSmallAlbums(QList<QVector<QString> *> *entries)
{
	QList<SmallAlbum*> *albums;
	if (entries->count() > 0)
	{
		albums = new QList<SmallAlbum*>();
		for (auto &&item : *entries)
		{
			SmallAlbum *album = getSmallAlbum(item);
			albums->append(album);
		}
	}
	else { albums = nullptr; }

	return albums;
}

/**
 * @brief changes a @see Album to a string-vector
 * @param album the given album
 * @return in representation needed for the database
 */
QMap<QString,QString>& DataWorker::getAlbumMapForDatabase(Album *album)
{
	QMap<QString,QString> *result = new QMap<QString,QString>();

	QString titleEncoded = album->title;
	Umlaute_Web::Qt::encode(titleEncoded);

	QString interpretEncoded = album->interpret;
	Umlaute_Web::Qt::encode(interpretEncoded);

	QString genreEncoded = album->grene;
	Umlaute_Web::Qt::encode(genreEncoded);

	QString locationEncoded = album->location;
	Umlaute_Web::Qt::encode(locationEncoded);

	QString ownerEncoded = album->owner;
	Umlaute_Web::Qt::encode(ownerEncoded);


	result->insert("Title", titleEncoded);
	result->insert("Interpret", interpretEncoded);
	result->insert("Year", QString::number(album->year));
	result->insert("EAN", album->ean);
	result->insert("Genre", genreEncoded);
	result->insert("Type", album->isCD_DVD ? "CD + DVD" : "CD");
	result->insert("Location", locationEncoded);
	result->insert("Owner", ownerEncoded);

	return *result;
}

/**
 * @brief converts a single string array to an @see Title
 * @param entry single database entry
 * @return \p entry as @see Title
 */
Title* DataWorker::getTitle(QVector<QString> *entry)
{
	bool parsingOK = true;
	Umlaute_Web::Qt::decode((*entry)[1]);	// Title
	Umlaute_Web::Qt::decode((*entry)[2]);	// Interpret
	Umlaute_Web::Qt::decode((*entry)[3]);	// Album
	Umlaute_Web::Qt::decode((*entry)[4]);	// Genre
	Umlaute_Web::Qt::decode((*entry)[6]);	// Owner
	Umlaute_Web::Qt::decode((*entry)[7]);	// Format

	short id = entry->at(0).toShort(&parsingOK);

	bool yearParsingOk = true;
	int year = entry->at(5).toInt(&yearParsingOk);

	Title *title = (parsingOK && yearParsingOk) ? new Title
	{
		id,
		entry->at(1),	// Title
		entry->at(2),	// Interpret
		entry->at(3),	// Album
		entry->at(4),	// Grene
		year,
		entry->at(6),	// Owner
		entry->at(7)	// Format
	} : nullptr;

	return title;
}

/**
 * @brief Convert a single string vector to an \see SmallTitle
 * @param entry The single database entry
 * @return \p entry as \see SmallTitle
 *
 * It must be ensured that \p entry has at least four entries and they can be interpreted as id, title, interpret and
 * album (in this order).
 */
SmallTitle* DataWorker::getSmallTitle(QVector<QString> *entry)
{
	bool parsingOK = true;
	short id = entry->at(0).toShort(&parsingOK);
	Umlaute_Web::Qt::decode((*entry)[1]);	// Title
	Umlaute_Web::Qt::decode((*entry)[2]);	// Interpret
	Umlaute_Web::Qt::decode((*entry)[3]);	// Album

	SmallTitle *title = new SmallTitle(id, entry->at(1), entry->at(3), entry->at(2));
	return title;
}

/**
 * @brief Converts string vector from databse to list of @see Title
 * @param entries Entries given from database
 * @return \p entries as @see Title
 */
QList<Title*>* DataWorker::getTitles(QList<QVector<QString>*> *entries)
{
	QList<Title*> *titles;
	if (entries->count() > 0)
	{
		titles = new QList<Title*>();
		for (auto &&item : *entries)
		{
			Title *title = getTitle(item);
			titles->append(title);
		}
	}
	else { titles = nullptr; }

	return titles;
}

/**
 * @brief Converts string vector from databse to list of @see SmallTitle
 * @param entries Entries given from database
 * @return \p entries as @see Title
 */
QList<SmallTitle*>* DataWorker::getSmallTitles(QList<QVector<QString>*> *entries)
{
	QList<SmallTitle*> *titles;
	if (entries->count() > 0)
	{
		titles = new QList<SmallTitle*>();
		for (auto &&item : *entries)
		{
			SmallTitle *title = getSmallTitle(item);
			titles->append(title);
		}
	}
	else { titles = nullptr; }

	return titles;
}

/**
 * @brief changes a  @see Title to a string-vector
 * @param title the given title
 * @return \p title in representation needed for the database
 */
QMap<QString,QString>& DataWorker::getTitleMapForDatabase(Title *title)
{
	QMap<QString,QString> *result = new QMap<QString,QString>();

	QString titleEncoded = title->title;
	Umlaute_Web::Qt::encode(titleEncoded);

	QString interpretEncoded = title->interpret;
	Umlaute_Web::Qt::encode(interpretEncoded);

	QString albumEncoded = title->album;
	Umlaute_Web::Qt::encode(albumEncoded);

	QString genreEncoded = title->grene;
	Umlaute_Web::Qt::encode(genreEncoded);

	QString ownerEncoded = title->owner;
	Umlaute_Web::Qt::encode(ownerEncoded);

	QString formatEncoded = title->format;
	Umlaute_Web::Qt::encode(formatEncoded);

	// iff title->id is -1 we want to create a new entry and therefore have to leave the ID filed in database empty
	result->insert("ID", title->id == -1 ? "NULL" : QString::number(title->id));
	result->insert("Title", titleEncoded);
	result->insert("Interpret", interpretEncoded);
	result->insert("Album", albumEncoded);
	result->insert("Genre", genreEncoded);
	result->insert("Year", QString::number(title->year));
	result->insert("Owner", ownerEncoded);
	result->insert("Fromat", formatEncoded);

	return *result;
}

/**
 * @brief Create the database column name out of the index of one "search combo box" for an album search
 * @param index The index of one combo box
 * @return The correcponding column name or an empty string
 */
QString DataWorker::createSearchCriterionForAlbum(int index)
{
	switch (index)
	{
		case 0:
			return "Title";
		case 1:
			return "Interpret";
		case 2:
			return "Year";
		case 3:
			return "EAN";
		case 4:
			return "Genre";
		case 5:
			return "Type";
		case 6:
			return "Ort";
		case 7:
			return "Owner";
		default:
			return "";
	}
}

/**
 * @brief Create the database column name out of the index of one "search combo box" for an title search
 * @param index The index of one combo box
 * @return The correcponding column name or an empty string
 */
QString DataWorker::createSearchCriterionForTitle(int index)
{
	switch (index)
	{
		case 0:
			return "ID";
		case 1:
			return "Title";
		case 2:
			return "Interpret";
		case 3:
			return "Album";
		case 4:
			return "Genre";
		case 5:
			return "Year";
		case 6:
			return "Owner";
		case 7:
			return "Format";
		default:
			return "";
	}
}

/**
 * @brief Create the filter map for an (mulit criteria) album search
 * @param index1 The index of the 1st ComboBox
 * @param index2 The index of the 2nd ComboBox
 * @param text1 The text of the first textbox
 * @param text2 The text of the second textbox
 * @return The filter map created out of the parameters or <tt>nullptr</tt> iff no criteria is selected or no search
 *		   text is given
 */
QMap<QString, QString>* DataWorker::createSearchFilterForAlbum(int index1, int index2, QString text1, QString text2)
{
	QMap<QString,QString> *searchFilter = nullptr;

	if (index1 > -1 && text1 != "")
	{
		if (searchFilter == nullptr) { searchFilter = new QMap<QString,QString>(); }
		searchFilter->insert(createSearchCriterionForAlbum(index1), text1);
	}
	if (index2 > -1 && text2 != "")
	{
		if (searchFilter == nullptr) { searchFilter = new QMap<QString,QString>(); }
		searchFilter->insert(createSearchCriterionForAlbum(index2), text2);
	}

	return searchFilter;
}

/**
 * @brief Create the filter map for an (mulit criteria) title search
 * @param index1 The index of the 1st ComboBox
 * @param index2 The index of the 2nd ComboBox
 * @param text1 The text of the first textbox
 * @param text2 The text of the second textbox
 * @return The filter map created out of the parameters or <tt>nullptr</tt> iff no criteria is selected or no search
 *		   text is given
 */
QMap<QString, QString> *DataWorker::createSearchFilterForTitle(int index1, int index2, QString text1, QString text2)
{
	QMap<QString,QString> *searchFilter = nullptr;

	if (index1 > -1 && text1 != "")
	{
		if (searchFilter == nullptr) { searchFilter = new QMap<QString,QString>(); }
		searchFilter->insert(createSearchCriterionForTitle(index1), text1);
	}
	if (index2 > -1 && text2 != "")
	{
		if (searchFilter == nullptr) { searchFilter = new QMap<QString,QString>(); }
		searchFilter->insert(createSearchCriterionForTitle(index2), text2);
	}

	return searchFilter;
}

