#include "server.h"

#include "credentialdialog.h"
#include "definitions.h"
#include "keychain.h"

#include <cstdlib>

#include <SqlWorker/DatabaseWorker.hpp>

#include <QApplication>
#include <QDialog>
#include <QEventLoop>
#include <QMessageBox>

#include <qt5keychain/keychain.h>

#ifdef QT_DEBUG
#include <iostream>
#endif

bool Server::allOk = true;

/**
 * @brief the one and only @see Server instance
 */
const Server* Server::instance = nullptr;

/**
 * @brief initializing the @see MySQLWorker::DatabaseWorker
 */
Server::Server(QString &serverAddress, QString &database, QString &username, QString &password)
{
	SqlWorker::DatabaseType type = SqlWorker::DatabaseType::MYSQL;
	this->database = new SqlWorker::DatabaseWorker(type, serverAddress, database, username, password);
}

/**
 * @brief delete all pointer references
 */
Server::~Server()
{
	delete database;
}

void Server::showCredentialError()
{
	QMessageBox::critical(
				nullptr,
				QApplication::translate("Definitions", PRODUCT_NAME),
				QApplication::translate("Definitions", "No credentials available.\nThe programme will be terminated.")
	);
	std::exit(-5);
}

bool Server::tryCreateInstance()
{
	QString *server = new QString("");
	QString *database = new QString("");
	QString *user = new QString("");
	QString *password = new QString("");

	//check credential store

	bool readingOk = true;

	QKeychain::ReadPasswordJob readDatabase(KeyChain::KEYCHAIN_NAMESPACE);
	QEventLoop databaseLoop;
	readDatabase.setAutoDelete(false);
	readDatabase.setKey(KeyChain::database_key);
	QObject::connect(
				&readDatabase,
				&QKeychain::ReadPasswordJob::finished,
				&databaseLoop,
				[&readDatabase, database, &readingOk, &databaseLoop]()
	{
		if (readDatabase.error())
		{
			readingOk &= false;
#ifdef QT_DEBUG
			std::cerr << "Read database failed: " << qPrintable(readDatabase.errorString()) << std::endl;
#endif
		}
		else
		{
			database->clear();
			database->append(readDatabase.textData());
			readingOk &= true;
		}
		databaseLoop.exit();
	});
	readDatabase.start();
	databaseLoop.exec();

	QKeychain::ReadPasswordJob readPassword(KeyChain::KEYCHAIN_NAMESPACE);
	QEventLoop passwordLoop;
	readPassword.setAutoDelete(false);
	readPassword.setKey(KeyChain::password_key);
	QObject::connect(
				&readPassword,
				&QKeychain::ReadPasswordJob::finished,
				&passwordLoop,
				[&readPassword, password, &readingOk, &passwordLoop]()
	{
		if (readPassword.error())
		{
			readingOk &= false;
#ifdef QT_DEBUG
			std::cerr << "Read password failed: " << qPrintable(readPassword.errorString()) << std::endl;
#endif
		}
		else
		{
			password->clear();
			password->append(readPassword.textData());
			readingOk &= true;
		}
		passwordLoop.exit();
	});
	readPassword.start();
	passwordLoop.exec();

	QKeychain::ReadPasswordJob readServer(KeyChain::KEYCHAIN_NAMESPACE);
	QEventLoop serverLoop;
	readServer.setAutoDelete(false);
	readServer.setKey(KeyChain::server_key);
	QObject::connect(
				&readServer,
				&QKeychain::ReadPasswordJob::finished,
				&serverLoop,
				[&readServer, server, &readingOk, &serverLoop](){
		if (readServer.error())
		{
			readingOk &= false;
#ifdef QT_DEBUG
			std::cerr << "Read server failed: " << qPrintable(readServer.errorString()) << std::endl;
#endif
		}
		else
		{
			server->clear();
			server->append(readServer.textData());
			readingOk &= true;
		}
		serverLoop.exit();
	});
	readServer.start();
	serverLoop.exec();

	QKeychain::ReadPasswordJob readUser(KeyChain::KEYCHAIN_NAMESPACE);
	QEventLoop userLoop;
	readUser.setAutoDelete(false);
	readUser.setKey(KeyChain::username_key);
	QObject::connect(
				&readUser,
				&QKeychain::ReadPasswordJob::finished,
				&userLoop,
				[&readUser, user, &readingOk, &userLoop]()
	{
		if (readUser.error())
		{
			readingOk &= false;
#ifdef QT_DEBUG
			std::cerr << "Read username failed: " << qPrintable(readUser.errorString()) << std::endl;
#endif
		}
		else
		{
			user->clear();
			user->append(readUser.textData());
			readingOk &= true;
		}
		userLoop.exit();
	});
	readUser.start();
	userLoop.exec();

//	loop.exec();

	if (readingOk)
	{
		// all reading was ok -> create the instance
		instance = new Server(*server, *database, *user, *password);
	}
	allOk = readingOk;
	return allOk;
}


/**
 * @brief get the one and only instance of the @see Server class
 * @return
 */
const Server& Server::getInstance()
{
	/* iff there is no instance try to get the information
	 * from the credential storage. If it has no information in it,
	 * show CredentialForm and get it from there.
	 */
	if (instance == nullptr)
	{
		if (!tryCreateInstance())
		{
			CredentialDialog credDialog{};
			if (credDialog.exec() == QDialog::Accepted)
			{
				/* User have not save the credentials and we could not get them from
				 * the keycain -> we have to abort everything (and inform the user about
				 * that).
				 */
				if (!tryCreateInstance())
				{
					showCredentialError();
				}
			}
			else
			{
				/* User have not save the credentials and we could not get them from
				 * the keycain -> we have to abort everything (and inform the user about
				 * that).
				 */
				showCredentialError();
			}
		}
	}
	return *instance;
}
