#include "titledetail.h"
#include "ui_titledetail.h"

#include "dataworker.h"
#include "definitions.h"
#include "server.h"

#include <QList>
#include <QMessageBox>
#include <QSet>
#include <QString>
#include <QVector>

/**
 * @brief constructor
 * @param parent the parent window
 */
TitleDetail::TitleDetail(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::TitleDetail),
	settings()
{
	ui->setupUi(this);

	int size = settings.beginReadArray("owners");
	for (int var = 0; var < size; ++var)
	{
		settings.setArrayIndex(var);
		ui->comboOwn->addItem(settings.value("owner").toString());
	}
	settings.endArray();
}

/**
 * @brief destructor, destruct ui
 */
TitleDetail::~TitleDetail()
{
	delete ui;
}

void TitleDetail::closeEvent(QCloseEvent *event)
{
	QSet<QString> owners;
	for (int i = 0; i < ui->comboOwn->count(); ++i)
	{
		QString owner = ui->comboOwn->itemText(i).simplified();
		if (owner != "") { owners.insert(owner); }
	}
	QString currentText = ui->comboOwn->currentText().simplified();
	if (currentText != "") { owners.insert(currentText); }

	settings.beginWriteArray("owners");
	int owner_counter = 0;
	foreach (QString elem, owners)
	{
		settings.setArrayIndex(owner_counter);
		settings.setValue("owner", elem);
		owner_counter++;
	}
	settings.endArray();

	QMainWindow::closeEvent(event);
}


/**
 * @brief adds a new entry to the database
 */
void TitleDetail::addEntry()
{
	QMap<QString,QString> databaseEntry;
	if (tryCreateDatabaseEntry(databaseEntry))
	{
		bool ok = Server::getInstance().database->insertNewEntry(TABLE, databaseEntry);
		if (ok)
		{
			QMessageBox::information(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						QApplication::translate("Definitions", INSERT_SUCESS)
			);
			this->close();
		}
		else
		{
			QMessageBox::critical(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						Server::getInstance().database->DatabaseWorker::getLastError()
			);
		}
	}
}

/**
 * @brief close this window
 */
void TitleDetail::btnBackClick()
{
	this->close();
}

/**
 * @brief saves changes on a entry to @see TABLE
 */
void TitleDetail::changeEntry()
{
	QMap<QString,QString> databaseEntry;
	if (tryCreateDatabaseEntry(databaseEntry))
	{
		QMap<QString,QString> filter{};
		filter.insert("ID", ui->txtID->text());

		bool ok = Server::getInstance().database->updateEntries(TABLE, filter, databaseEntry);
		if (ok)
		{
			QMessageBox::information(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						QApplication::translate("Definitions", CHANGE_SUCCESS)
			);
			this->close();
		}
		else
		{
			QMessageBox::critical(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						Server::getInstance().database->DatabaseWorker::getLastError()
			);
		}
	}
}

/**
 * @brief prepares window for 'view entry only' mode
 */
void TitleDetail::disableAll()
{
	ui->btnAdd->setEnabled(false);
	ui->btnBack->setDefault(true);
	ui->btnSubmit->setEnabled(false);
	ui->comboOwn->setEnabled(false);
	ui->txtAlbum->setReadOnly(true);
	ui->txtCodec->setReadOnly(true);
	ui->txtGrene->setReadOnly(true);
	ui->txtID->setReadOnly(true);
	ui->txtIpret->setReadOnly(true);
	ui->txtTitle->setReadOnly(true);
	ui->txtYear->setReadOnly(true);
}

/**
 * @brief prepares window for 'add entry' mode
 */
void TitleDetail::enableAll()
{
	ui->btnAdd->setDefault(true);
	ui->btnAdd->setEnabled(true);
	ui->btnSubmit->setEnabled(false);
	ui->comboOwn->setEnabled(true);
	ui->txtAlbum->setReadOnly(false);
	ui->txtCodec->setReadOnly(false);
	ui->txtGrene->setReadOnly(false);
	ui->txtID->setReadOnly(false);
	ui->txtIpret->setReadOnly(false);
	ui->txtTitle->setReadOnly(false);
	ui->txtYear->setReadOnly(false);
}

/**
 * @brief gets an entry from the database
 * @param id identifier [primary key] of the searched entry
 */
void TitleDetail::getEntry(int id)
{
	QMap<QString,QString> filter{};
	filter.insert(PRIMARY_KEY, QString::number(id));

	QList<QVector<QString>*> *dbValue =
			Server::getInstance().database->searchEntries(TABLE, &filter, nullptr, PRIMARY_KEY);

	if (dbValue != nullptr && dbValue->count() > 0)
	{
		Title *title = DataWorker::getTitle(dbValue->first());
		if (title != nullptr)
		{
			output(title);
			delete title;
		}
		else
		{
			QMessageBox::critical(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						QApplication::translate("Definitions", DATA_READ_ERROR)
			);
		}
		delete dbValue;
	}
	else
	{
		QMessageBox msgbox;
		msgbox.setText(QApplication::translate("Definitions", DB_READ_ERROR));
		msgbox.setInformativeText(Server::getInstance().database->DatabaseWorker::getLastError());
		msgbox.setIcon(QMessageBox::Critical);
		msgbox.exec();
	}

}

/**
 * @brief set values based on \p id, showing window
 * @param id id of the entry to select from database
 * @param mode defines how the window should look like
 */
void TitleDetail::load(short id, DetailModi mode)
{
	switch (mode)
	{
		case DetailModi::MODE_ADD_ENTRY:
			enableAll();
			break;

		case DetailModi::MODE_READONLY_ALL:
			getEntry(id);
			disableAll();
			break;

		case DetailModi::MODE_READONLY_IDENTIFYER:
			getEntry(id);
			setIdReadOnly();
			break;

		default:
			getEntry(id);
			break;
	}
	show();
}

/**
 * @brief prints all @see Title inforamtions
 * @param title the given title
 */
void TitleDetail::output(Title *title)
{
	if (title != nullptr)
	{
		ui->comboOwn->setCurrentText(title->owner);
		ui->txtAlbum->setText(title->album);
		ui->txtCodec->setText(title->format);
		ui->txtGrene->setText(title->grene);
		ui->txtID->setText(QString::number(title->id));
		ui->txtIpret->setText(title->interpret);
		ui->txtTitle->setText(title->title);
		ui->txtYear->setText(QString::number(title->year));
	}
}

/**
 * @brief prepares window for 'edit entry' mode
 */
void TitleDetail::setIdReadOnly()
{
		ui->txtAlbum->setReadOnly(false);
		ui->txtGrene->setReadOnly(false);
		ui->txtIpret->setReadOnly(false);
		ui->txtCodec->setReadOnly(false);
		ui->txtTitle->setReadOnly(false);
		ui->txtYear->setReadOnly(false);
		ui->comboOwn->setEnabled(true);
		ui->btnSubmit->setEnabled(true);
		ui->btnAdd->setEnabled(false);
		ui->btnSubmit->setDefault(true);
}

/**
 * @brief creates the string-array for database out of window-control values
 * @param databaseEntry filled string-vector for database
 * @return <tt>true</tt> if the creation of an database-entry was successfull, <tt>false</tt> otherwise
 */
bool TitleDetail::tryCreateDatabaseEntry(QMap<QString,QString> &databaseEntry)
{
	bool idOk = true;
	QString idString = ui->txtID->text();
	short id = idString != "" ? idString.toShort(&idOk): -1;

	bool yearOk = true;
	int year = ui->txtYear->text().toShort(&yearOk);

	if (!(idOk && yearOk))
	{
		QMessageBox::critical(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					QApplication::translate("Definitions", CREATION_FAILED)
		);
		return false;
	}

	Title title(
				id,
				ui->txtTitle->text(),
				ui->txtIpret->text(),
				ui->txtAlbum->text(),
				ui->txtGrene->text(),
				year,
				ui->comboOwn->currentText().simplified(),
				ui->txtCodec->text());
	databaseEntry = DataWorker::getTitleMapForDatabase(&title);
	return true;
}
